


/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SpriteParticleRenderer.h"

float causize = 10;
float cauintens = 40;

// Listener class for frame updates
class RaytraceDemoListener : public ExampleFrameListener
{
protected:
	SceneNode* goldSphereNode;
	SceneNode* silverSphereNode;
	SceneNode* copperSphereNode;
	SceneNode* aluSphereNode;
	SceneNode* lightNode;
	unsigned long framecount;
	
public:
    RaytraceDemoListener(RenderWindow* window, Camera* maincam,SceneNode* lightNode,
																SceneNode* goldSphereNode,
																SceneNode* silverSphereNode,
																SceneNode* copperSphereNode,
																SceneNode* aluSphereNode)
        :ExampleFrameListener(window, maincam) 
				 
    {
		framecount = 0;
		this->goldSphereNode = goldSphereNode;
		this->silverSphereNode = silverSphereNode;
		this->copperSphereNode = copperSphereNode;
		this->aluSphereNode = aluSphereNode;
		this->lightNode = lightNode;
    }
	bool processUnbufferedKeyInput(const FrameEvent& evt)
    {
		MaterialPtr mat = MaterialManager::getSingleton().getByName("GameTools/Cauathene_SE_0_clone");
		GpuProgramParameters* Vparams = mat->getTechnique(0)->getPass(0)->getVertexProgramParameters().getPointer();
		GpuProgramParameters* Fparams = mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters().getPointer();
		
		if (mInputDevice->isKeyDown(KC_SUBTRACT))
        {
			cauintens -= 0.5;
			cauintens = std::max(cauintens, 1.0f);
			float intensity = pow( cauintens / 100.0f, 2 );
			Fparams->setNamedConstant("CauSpriteIntens", intensity);			
		}
		if (mInputDevice->isKeyDown(KC_ADD))
        {
			cauintens += 0.5;
			cauintens = std::min(cauintens, 100.0f);
			float intensity = pow( cauintens / 100.0f, 2 );
			Fparams->setNamedConstant("CauSpriteIntens", intensity);				
		}
		if (mInputDevice->isKeyDown(KC_MULTIPLY))
        {
			causize = std::max(causize - 0.1, 0.0);
			Vparams->setNamedConstant("CauSpriteSize", causize);			
		}
		if (mInputDevice->isKeyDown(KC_DIVIDE))
        {
			causize = causize + 0.1;
			Vparams->setNamedConstant("CauSpriteSize", causize);			
		}

		return ExampleFrameListener::processUnbufferedKeyInput(evt);
	}
    bool frameStarted(const FrameEvent& evt)
    {
		static bool first = true;
		if(first)
		{
			MaterialPtr mat = MaterialManager::getSingleton().getByName("GameTools/Cauathene_SE_0_clone");
			GpuProgramParameters* Vparams = mat->getTechnique(0)->getPass(0)->getVertexProgramParameters().getPointer();
			GpuProgramParameters* Fparams = mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters().getPointer();
			float intensity = pow( cauintens / 100.0f, 2 );
			Fparams->setNamedConstant("CauSpriteIntens", intensity);			
			Vparams->setNamedConstant("CauSpriteSize", causize);
			first = false;
		}

		lightNode->setPosition( 100.0 * Vector3(cos(-(float)framecount/500.0),2.0,sin(-(float)framecount/500.0)));
		//lightNode->setDirection(-lightNode->getPosition());
		
		goldSphereNode->setPosition(   100.0 * 
								Vector3(
										cos((float)framecount/700.0),
										1.0,
										sin((float)framecount/700.0)));
		silverSphereNode->setPosition( 100.0 * 
								Vector3(
										cos((float)framecount/700.0 + Math::PI / 2.0),
										1.0,
										sin((float)framecount/700.0 + Math::PI / 2.0)));
		copperSphereNode->setPosition( 100.0 * 
								Vector3(
										cos((float)framecount/700.0 + Math::PI ),
										1.0,
										sin((float)framecount/700.0 + Math::PI)));
		aluSphereNode->setPosition(    100.0 * 
								Vector3(
										cos((float)framecount/700.0 + 3 * Math::PI / 2.0), 
										1.0,
										sin((float)framecount/700.0+ 3 * Math::PI / 2.0)));
		
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;
      return result;     
    }

	

};

/** Application class */
class RaytraceDemoApplication : public ExampleApplication
{

public:
	virtual void go(void)
    {
        if (!setup())
            return;

        renderScene();
		
        // clean up
        destroyScene();
    }


protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
	SceneNode* goldSphereNode;
	SceneNode* silverSphereNode;
	SceneNode* copperSphereNode;
	SceneNode* aluSphereNode;

void renderScene(){
		
		mWindow->resetStatistics();
		while(true){		

			
			#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32            
			// Pump events on Win32
    		MSG  msg;
			while( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
			
			#endif
			
			if(!mRoot->_fireFrameStarted()){
				break;
			}
			unsigned long framenum = mRoot->getCurrentFrameNumber();
			
			OgreIlluminationManager::getSingleton().update(framenum, mWindow);
			
			//mRoot->renderOneFrame();
			mRoot->_updateAllRenderTargets();

			mRoot->_fireFrameEnded();	
			
		}
	}

	void createPostproc()
	{
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/Glow", true);
/*
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/ToneMap", true);*/
	}
    

	void createScene(void)
    {

		createPostproc();
		
		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		
		OgreIlluminationManager::getSingleton().setMaxJoinRadius(420);		

		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);

		mCamera->setPosition(0,100,240);
		mCamera->setFOVy(Radian(Degree(80)));
		//mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(1, 1, 1));

		// Skybox
        mSceneMgr->setSkyBox(true, "Examples/MorningSkyBox");

		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

      
		//init objects
		Entity* kupola = mSceneMgr->createEntity("kupola", "kupola.mesh");
		Entity* athene = mSceneMgr->createEntity("athene", "head.mesh");
		Entity* goldSphere = mSceneMgr->createEntity("goldSphere", "head.mesh");
		Entity* silverSphere = mSceneMgr->createEntity("silverSphere", "head.mesh");
		Entity* copperSphere = mSceneMgr->createEntity("copperSphere", "head.mesh");
		Entity* aluSphere = mSceneMgr->createEntity("aluSphere", "head.mesh");

		kupola->setCastShadows(false);
		//lod
		//MeshPtr amesh = athene->getMesh();
		//Mesh::LodDistanceList distances;
		//distances.push_back(1);
		//distances.push_back(2);
		//distances.push_back(3);
		//distances.push_back(4);
		////distances.push_back(5);
		//amesh->generateLodLevels(distances,ProgressiveMesh::VertexReductionQuota::VRQ_PROPORTIONAL,0.6);
		//athene->setMeshLodBias(50000.0,0);
		//tangents
		//amesh->buildTangentVectors(0,1);
		MeshPtr kmesh = kupola->getMesh();
		kmesh->buildTangentVectors(0,1);
		
		//init nodes
		kupolaNode = rootNode->createChildSceneNode( "kupola" );
		kupolaNode->rotate(Vector3(0,1,0),Radian(Degree(90)));
		kupolaNode->attachObject( kupola );
		kupolaNode->scale(25,21,25);		
		//kupolaNode->scale(2,2,2);		
		//kupolaNode->setPosition(0,100,0);
		SceneNode* atheneNode = rootNode->createChildSceneNode("athene");
		atheneNode->attachObject(athene);
		//atheneNode->setScale(3.5,5.5,5.5);
		atheneNode->setScale(4,4,4);
		//atheneNode->setScale(0.5,0.5,0.5);
		AxisAlignedBox bbox = athene->getBoundingBox();
		atheneNode->setPosition(0, 100,0);
		

		goldSphereNode = rootNode->createChildSceneNode( "goldSphere" );
		//goldSphereNode->scale(0.3,0.3,0.3);
		goldSphereNode->scale(2,2,2);
		silverSphereNode = rootNode->createChildSceneNode( "silverSphere" );
		//silverSphereNode->scale(0.3,0.3,0.3);
		silverSphereNode->scale(2,2,2);
		copperSphereNode = rootNode->createChildSceneNode( "copperSphere" );
		//copperSphereNode->scale(0.3,0.3,0.3);
		copperSphereNode->scale(2,2,2);
		aluSphereNode = rootNode->createChildSceneNode( "aluSphere" );
		//aluSphereNode->scale(0.3,0.3,0.3);
		aluSphereNode->scale(2,2,2);

		goldSphereNode->attachObject(goldSphere);
		silverSphereNode->attachObject(silverSphere);
		aluSphereNode->attachObject(aluSphere);
		copperSphereNode->attachObject(copperSphere);

		goldSphereNode->_updateBounds();
		silverSphereNode->_updateBounds();
		aluSphereNode->_updateBounds();
		copperSphereNode->_updateBounds();

		kupolaNode->_updateBounds();
		atheneNode->_updateBounds();

		//Init lights
		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_POINT);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		mainLightNode->setPosition(0,0,0);		
		mainLight->setPosition(mainLightNode->getWorldPosition());
		mainLightNode->attachObject(mainLight); 	

		aluSphere->setMaterialName("EnvMetals/Alu");
		copperSphere->setMaterialName("EnvMetals/Copper");
		
		{
			Material* oldmat = (Material*) MaterialManager::getSingleton().getByName("EnvMetals/Gold").getPointer();
			Material* newmat = oldmat->clone("gold1").getPointer();
			Material* newmat2 = oldmat->clone("gold2").getPointer();

			goldSphere->setMaterialName(newmat->getName());
			silverSphere->setMaterialName(newmat2->getName());
		}

		athene->setMaterialName("GameTools/CausticGlass");
		//athene->setMaterialName("GameTools/CubeMap/Diffuse");
		
		OgreIlluminationManager::getSingleton().initTechniques();
		
		
   }

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new RaytraceDemoListener(mWindow,
											mCamera,
											mainLightNode,
											goldSphereNode,
											silverSphereNode,
											copperSphereNode,
											aluSphereNode);
        mRoot->addFrameListener(mFrameListener);
		//mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());

    }

};

